@@ -1,6 +1,6 @@
 """Typer, build great CLIs. Easy to code. Based on Python type hints."""
 
-__version__ = "0.12.1"
+__version__ = "0.13.1"
 
 from shutil import get_terminal_size as get_terminal_size
 
@@ -12,7 +12,6 @@ from click.termui import confirm as confirm
 from click.termui import echo_via_pager as echo_via_pager
 from click.termui import edit as edit
 from click.termui import getchar as getchar
-from click.termui import launch as launch
 from click.termui import pause as pause
 from click.termui import progressbar as progressbar
 from click.termui import prompt as prompt
@@ -28,6 +27,7 @@ from click.utils import open_file as open_file
 
 from . import colors as colors
 from .main import Typer as Typer
+from .main import launch as launch
 from .main import run as run
 from .models import CallbackParam as CallbackParam
 from .models import Context as Context
